#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define INFINITE -1

#define SENSOR_PIN PB0
#define SENSOR_REG_INTR PCIE1
#define SENSOR_INTR PCINT8

#define MOTOR1_FP PB4
#define MOTOR1_BP PC3
#define MOTOR2_FP PB3
#define MOTOR2_BP PC1 
						/* motor1 is on the right 
						   motor2 is on the left*/


volatile int state = 1; // 1 - forward
int starting_speed = 2450;
volatile int speed = 2450;

void go_forward(int speed);
void go_backward(int speed);
void rotate_left();
void init();
void stop();

ISR(PCINT1_vect)
{
	if ( (PINB & (1 << PB0)) == 0) // something detected
	{
		/* stop timer */
		TCCR1B &= 248;
		/* clear counter */
		TCNT1 = 0;

		state = 2; // go backward
	}
}

ISR(TIMER2_COMPA_vect)
{
	speed += 1;
	if (speed > 2550)
		speed = 2550;

}


ISR(TIMER1_COMPA_vect)
{
	
	/* stop timer */
	TCCR1B &= 248;
	/* clear counter */
	TCNT1 = 0;
	if (state == -2)
		state = 3;
	else if (state == -3)
	{
		state = 1;

	}
	speed = starting_speed;
}

int main(void) {

	int current_state;
	int prev_state = 0;
	cli();
	init();
	sei();
	
	while(1)
	{
		prev_state = current_state;
		current_state = state;

		if (current_state == 1)
		{
			/* start timer */
			TCCR2B = (7 << CS20);
			go_forward(speed/10);
			
		}
		else if (current_state == 2)
		{
			/* start timer */
			TCCR1B = (5 << CS10);
			
			TCCR2B &= 248;
			go_backward(255);
			state = -2; 
			
		}
		else if (current_state == 3)
		{
		
			/* start timer */
			TCCR1B = (5 << CS10);
			TCCR2B &= 248;
			rotate_left();

			state = -3; 
			
		}
		
	}
	return 0;
}




void init()
{
	/* PB0 as input */
	DDRB &= ~(1 << SENSOR_PIN);
	/* Pull-up resistance on PB0 */
	PORTB |= (1 << SENSOR_PIN);
	
	
	/*  outputs */
	DDRB |= (1 << MOTOR1_FP);
	DDRC |= (1 << MOTOR1_BP);
	DDRB |= (1 << MOTOR2_FP);
	DDRC |= (1 << MOTOR2_BP);
	
	/* enable SENSOR REG intr */
	PCICR  |= (1 << SENSOR_REG_INTR);
	/* enable SENSOR intr */
    PCMSK1 |= (1 << SENSOR_INTR);
	
	/* use timer1 for counting */
	
	/* count one second */
	OCR1A = 15625;
	
	
	/* enable compare intr with OCR1A */
	TIMSK1 |= (1 << OCIE1A);
	TIMSK1 |= (1 << OCIE1B);
	
	/* CTC with OCR1A */
	TCCR1B |= (1 << WGM12);
	
	
	
	/* use timer0 for pwm */
	
	/* fast pwm */
	TCCR0A |= (3 << WGM00);
	
	
	TCCR0A |= (2 << COM0A0);
	TCCR0A |= (2 << COM0B0);
	
	/* no prescaler */
	TCCR0B |= (1 << CS00);
	
	OCR0A = 0;
	OCR0B = 0;
	
	
	/* use timer2 for accelerating */
	
	/* CTC with OCR1A */
	TCCR2A = (1 << WGM21);
	
	/* 1024 prescaler */
	TCCR2B = (7 << CS20);
	
	OCR2A = 56;
	
	TIMSK2 |= (1 << OCIE2A);
	
}

void go_forward(int speed)
{
	stop();
	
	
	OCR0A = speed;
	PORTC &= ~(1 << MOTOR1_BP); 
	

	OCR0B = speed;
	PORTC &= ~(1 << MOTOR2_BP); 
}

void go_backward(int speed)
{
	stop();
	
	OCR0A = 0;
	PORTC |=  (1 << MOTOR1_BP); 
		
	OCR0B = 0;
	PORTC |=  (1 << MOTOR2_BP); 
}

void stop()
{
	OCR0A = 0;
	PORTC &= ~(1 << MOTOR1_BP); 
	
	OCR0B = 0;
	PORTC &= ~(1 << MOTOR2_BP); 
}

void rotate_left()
{
	stop();
	
	
	OCR0A = 255;
	PORTC &= ~(1 << MOTOR1_BP); 
		/* right goes forward */
	  
	OCR0B = 0;
	PORTC |=  (1 << MOTOR2_BP); 
		/* left goes backward */
}


